@@ -1,5 +1,5 @@
 /************************************************************************************
-   Copyright (C) 2023 MariaDB Corporation AB
+   Copyright (C) 2023, 2025 MariaDB Corporation plc
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
@@ -590,8 +590,8 @@ namespace mariadb
     // check that queries are rewritable
     bool canAggregateSemiColumn= true;
     std::size_t totalLen= 0;
-    for (SQLString query : queries){
-      if (!ClientPrepareResult::canAggregateSemiColon(query,noBackslashEscapes())){
+    for (SQLString query : queries) {
+      if (!ClientPrepareResult::canAggregateSemiColon(query, noBackslashEscapes())) {
         canAggregateSemiColumn= false;
         break;
       }
@@ -1008,16 +1008,26 @@ namespace mariadb
     std::lock_guard<std::mutex> localScopeLock(lock);
     cmdPrologue();
 
-    //try {
     if (mysql_stmt_execute(serverPrepareResult->getStatementId()) != 0) {
       throwStmtError(serverPrepareResult->getStatementId());
     }
     /*CURSOR_TYPE_NO_CURSOR);*/
     getResult(results, serverPrepareResult);
-    //}
-    //  catch (SQLException& qex){
-    //  throw //logQuery->exceptionWithQuery(parameters, qex, serverPrepareResult);
-    //}
+  }
+
+  /* Direct exectution of server-side prepared statement */
+  void Protocol::directExecutePreparedQuery(
+      ServerPrepareResult* serverPrepareResult,
+      Results* results)
+  {
+    std::lock_guard<std::mutex> localScopeLock(lock);
+    cmdPrologue();
+    auto& query= serverPrepareResult->getSql();
+    if (mariadb_stmt_execute_direct(serverPrepareResult->getStatementId(), query.c_str(), query.length()) != 0) {
+      throwStmtError(serverPrepareResult->getStatementId());
+    }
+    /*CURSOR_TYPE_NO_CURSOR);*/
+    getResult(results, serverPrepareResult);
   }
 
   /** Rollback transaction. */
@@ -1056,15 +1066,19 @@ namespace mariadb
     if (!needLock || lock.try_lock()) {
       //cmdPrologue();
       // If connection is closed - we still need to call mysql_stmt_close to deallocate C/C stmt handle
-      if (mysql_stmt_close(statementId))
-      {
-        needLock && (lock.unlock(), true);
+      if (mysql_stmt_close(statementId)) {
+        if (needLock) {
+          lock.unlock();
+        }
         throw SQLException("Could not deallocate query");
       }
-      needLock && (lock.unlock(), true);
+      if (needLock) {
+        lock.unlock();
+      }
       return true;
 
-    } else {
+    }
+    else {
       statementIdToRelease= statementId;
     }
 
