@@ -1,5 +1,5 @@
 /************************************************************************************
-   Copyright (C) 2013,2019 MariaDB Corporation AB
+   Copyright (C) 2013,2024 MariaDB plc
    
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
@@ -19,14 +19,15 @@
 
 #include <Windows.h>
 #include <stdlib.h>
+#include <stdio.h>
 #include <CommCtrl.h>
 #include <tchar.h>
 #include <windowsx.h>
 #include <winuser.h>
 #include <shobjidl.h>
 #include "resource.h"
-#include <ma_odbc.h>
-#include <ma_odbc_setup.h>
+#include "ma_dsn.h"
+#include "ma_odbc_setup.h"
 
 #pragma comment(lib, "ComCtl32.lib")
 
@@ -46,6 +47,7 @@ HWND          hwndTab[7], hwndMain;
 const int    *EffectiveDisabledPages=    NULL,
              *EffectiveDisabledControls= NULL;
 BOOL          OpenCurSelection=          TRUE;
+char          tipText[256];
 
 /* On Windows we are supposed to have schannel, or we can have openssl */
 #if defined(_WIN32) || defined(HAVE_OPENSSL) 
@@ -91,6 +93,12 @@ MADB_DsnMap DsnMap[] = {
   {&DsnKeys[38], 3, ckSchParamNoError,    0, 0},
   {&DsnKeys[41], 3, ckNoLocalInfile,      0, 0},
   {&DsnKeys[42], 3, ckNullSchemaMeansCurrent, 0, 0},
+  {&DsnKeys[44], 3, txtAttr,            260, 0},
+  {&DsnKeys[45], 3, ckServerDirectExec,   0, 0},
+  {&DsnKeys[46], 3, ckClientPrepare,      0, 0},
+  {&DsnKeys[47], 3, txtPsCacheSize,       0, 0},
+  {&DsnKeys[48], 3, txtMaxCacheKey,       0, 0},
+  {&DsnKeys[49], 3, ckParamCallbacks,     0, 0},
   {&DsnKeys[19], 4, txtSslKey,          260, 0},
   {&DsnKeys[20], 4, txtSslCert,         260, 0},
   {&DsnKeys[21], 4, txtSslCertAuth,     260, 0},
@@ -104,6 +112,7 @@ MADB_DsnMap DsnMap[] = {
   {&DsnKeys[27], 4, txtCrl,               0, 0},
   {&DsnKeys[25], 4, txtTlsPeerFp,        41, 0},
   {&DsnKeys[26], 4, txtTlsPeerFpList,   260, 0},
+  {&DsnKeys[50], 5, ckResultCallbacks,    0, 0},
   {NULL, 0, 0, 0, 0}
 };
 
@@ -123,7 +132,6 @@ MADB_OptionsMap OptionsMap[]= {
   {3, ckMultiStmt,                     MADB_OPT_FLAG_MULTI_STATEMENTS},
   {LASTPAGE, ckEnableDynamicCursor,    MADB_OPT_FLAG_DYNAMIC_CURSOR},
   {LASTPAGE, ckDisableDriverCursor,    MADB_OPT_FLAG_NO_DEFAULT_CURSOR},
-  {LASTPAGE, ckDontCacheForwardCursor, MADB_OPT_FLAG_NO_CACHE},
   {LASTPAGE, ckForwardCursorOnly,      MADB_OPT_FLAG_FORWARD_CURSOR},
   {LASTPAGE, ckDontCacheForwardCursor, MADB_OPT_FLAG_NO_CACHE},
   {LASTPAGE, ckReturnMatchedRows,      MADB_OPT_FLAG_FOUND_ROWS},
@@ -182,13 +190,19 @@ my_bool SetDialogFields()
     case DSN_TYPE_BOOL:
       {
         my_bool Val= *(my_bool *)((char *)Dsn + DsnMap[i].Key->DsnOffset);
-        SendDlgItemMessage(hwndTab[DsnMap[i].Page], DsnMap[i].Item, BM_SETCHECK,
-                           Val ? BST_CHECKED : BST_UNCHECKED, 0);
+        if (Val != DSN_DEFAULT_TRUE)
+        {
+          SendDlgItemMessage(hwndTab[DsnMap[i].Page], DsnMap[i].Item, BM_SETCHECK,
+            Val ? BST_CHECKED : BST_UNCHECKED, 0);
+        }
       }
       break;
     case DSN_TYPE_CBOXGROUP:
       SendDlgItemMessage(hwndTab[DsnMap[i].Page], DsnMap[i].Item, BM_SETCHECK,
         (*GET_FIELD_PTR(Dsn, DsnMap[i].Key, char) & CBGROUP_BIT(i)) != '\0' ? BST_CHECKED : BST_UNCHECKED, 0);
+    case DSN_TYPE_RBGROUP:
+      SendDlgItemMessage(hwndTab[DsnMap[i].Page], DsnMap[i].Item, BM_SETCHECK,
+        (*GET_FIELD_PTR(Dsn, DsnMap[i].Key, char) == CBGROUP_BIT(i)) != '\0' ? BST_CHECKED : BST_UNCHECKED, 0);
     }
     i++;
   }
@@ -378,8 +392,14 @@ void GetDialogFields()
        *(int *)((char *)Dsn + DsnMap[i].Key->DsnOffset)= GetFieldIntVal(DsnMap[i].Page, DsnMap[i].Item);
        break;
     case DSN_TYPE_BOOL:
-      *GET_FIELD_PTR(Dsn, DsnMap[i].Key, my_bool)=  IS_CB_CHECKED(i);
+    {
+      my_bool *fieldPtr= GET_FIELD_PTR(Dsn, DsnMap[i].Key, my_bool), buttonState= IS_CB_CHECKED(i);
+      if (*fieldPtr != DSN_DEFAULT_TRUE || buttonState)
+      {
+        *fieldPtr=  buttonState;
+      }
       break;
+    }
     case DSN_TYPE_CBOXGROUP:
       if (IS_CB_CHECKED(i) != '\0')
       {
@@ -389,6 +409,11 @@ void GetDialogFields()
       {
         CBGROUP_RESETBIT(Dsn, i);
       }
+    case DSN_TYPE_RBGROUP:
+      if (IS_CB_CHECKED(i) != '\0')
+      {
+        *GET_FIELD_PTR(Dsn, DsnMap[i].Key, char)= CBGROUP_BIT(i);
+      }
     }
     ++i;
   }
@@ -402,9 +427,9 @@ void GetDialogFields()
   }
 }
 
-void DSN_Set_Database(SQLHANDLE Connection)
+void DSN_SetDatabase(SQLHANDLE Connection)
 {
-  MADB_Stmt *Stmt= NULL;
+  SQLHANDLE Stmt= NULL;
   SQLRETURN ret= SQL_ERROR;
   char Database[65];
   MADB_Dsn *Dsn= (MADB_Dsn *)GetWindowLongPtr(GetParent(hwndTab[0]), DWLP_USER);
@@ -416,10 +441,10 @@ void DSN_Set_Database(SQLHANDLE Connection)
 
   GetDialogFields();
   
-  if (SQLAllocHandle(SQL_HANDLE_STMT, Connection, (SQLHANDLE *)&Stmt) != SQL_SUCCESS)
+  if (SQLAllocHandle(SQL_HANDLE_STMT, Connection, &Stmt) != SQL_SUCCESS)
     goto end;
 
-  if (SQLExecDirect((SQLHSTMT)Stmt, (SQLCHAR *)"SHOW DATABASES", SQL_NTS) != SQL_SUCCESS)
+  if (SQLExecDirect(Stmt, (SQLCHAR *)"SHOW DATABASES", SQL_NTS) != SQL_SUCCESS)
     goto end;
 
   SQLBindCol(Stmt, 1, SQL_C_CHAR, Database, 65, 0);
@@ -438,12 +463,12 @@ void DSN_Set_Database(SQLHANDLE Connection)
 
 end:
   if (Stmt)
-	  SQLFreeHandle(SQL_HANDLE_STMT, (SQLHANDLE)Stmt);
+	  SQLFreeHandle(SQL_HANDLE_STMT, Stmt);
 }
 
-void DSN_Set_CharacterSets(SQLHANDLE Connection)
+void DSN_SetCharacterSets(SQLHANDLE Connection)
 {
-  MADB_Stmt *Stmt= NULL;
+  SQLHANDLE Stmt= NULL;
   SQLRETURN ret= SQL_ERROR;
   char Charset[65];
   MADB_Dsn *Dsn= (MADB_Dsn *)GetWindowLongPtr(GetParent(hwndTab[0]), DWLP_USER);
@@ -453,7 +478,7 @@ void DSN_Set_CharacterSets(SQLHANDLE Connection)
 
   GetDialogFields();
   
-  if (SQLAllocHandle(SQL_HANDLE_STMT, Connection, (SQLHANDLE *)&Stmt) != SQL_SUCCESS)
+  if (SQLAllocHandle(SQL_HANDLE_STMT, Connection, &Stmt) != SQL_SUCCESS)
     goto end;
 
   if (SQLExecDirect((SQLHSTMT)Stmt, 
@@ -480,7 +505,7 @@ void DSN_Set_CharacterSets(SQLHANDLE Connection)
 
 end:
   if (Stmt)
-	  SQLFreeHandle(SQL_HANDLE_STMT, (SQLHANDLE)Stmt);
+	  SQLFreeHandle(SQL_HANDLE_STMT, Stmt);
 }
 
 
@@ -509,7 +534,10 @@ static SQLRETURN TestDSN(MADB_Dsn *Dsn, SQLHANDLE *Conn, SQLCHAR *ConnStrBuffer)
   MADB_DsnToString(Dsn, ConnStr, CONNSTR_BUFFER_SIZE);
 
   SQLAllocHandle(SQL_HANDLE_DBC, Environment, (SQLHANDLE *)&Connection);
-  assert(Connection != NULL);
+  if (Connection == NULL)
+  {
+    return SQL_ERROR;
+  }
   Result= SQLDriverConnect(Connection, NULL, ConnStr, CONNSTR_BUFFER_SIZE, NULL, 0, NULL, SQL_DRIVER_NOPROMPT);
 
   Dsn->InitCommand= InitCommand;
@@ -541,7 +569,7 @@ char* HidePwd(char *ConnStr)
     char *KeyValBorder= strchr(Ptr, '=');
     char StopChr=        ';';
 
-    Ptr= ltrim(Ptr);
+    Ptr= (char*)ltrim(Ptr);
 
     if (_strnicmp(Ptr, "PWD", 3) == 0 || _strnicmp(Ptr, "PASSWORD", 8) == 0)
     {
@@ -549,7 +577,7 @@ char* HidePwd(char *ConnStr)
     }
     if (KeyValBorder != NULL)
     {
-      Ptr= ltrim(KeyValBorder + 1);
+      Ptr= (char*)ltrim(KeyValBorder + 1);
     }
     if (*Ptr == '{')
     {
@@ -599,20 +627,24 @@ void MADB_WIN_TestDsn(my_bool ShowSuccess)
       _snprintf(Info, sizeof(Info), "Connection successfully established\n\nServer Information: %s %s\n\nConnection String:\n\n%s", DbmsName, DbmsVer, ConnStr);
       MessageBox(hwndTab[CurrentPage], Info, "Connection test", MB_ICONINFORMATION| MB_OK);
     }
-    else
+    else if (Connection != NULL)
     {
       SQLCHAR SqlState[6], ErrMsg[SQL_MAX_MESSAGE_LENGTH];
       SQLGetDiagRec(SQL_HANDLE_DBC,   Connection, 1, SqlState, NULL, ErrMsg, SQL_MAX_MESSAGE_LENGTH, NULL);
       _snprintf(Info, sizeof(Info), "Connection failed: [%s] %s\n\nConnection String:\n\n%s", SqlState, ErrMsg, ConnStr);
       MessageBox(hwndTab[CurrentPage], Info, "Connection test", MB_ICONINFORMATION| MB_OK);
     }
+    else
+    {
+      MessageBox(hwndTab[CurrentPage], "Could not allocate connection handle", "Connection test", MB_ICONINFORMATION | MB_OK);
+    }
   }
 
   if (SQL_SUCCEEDED(ret))
   {
     ConnectionOK= TRUE;
-    DSN_Set_CharacterSets(Connection);
-    DSN_Set_Database(Connection);
+    DSN_SetCharacterSets(Connection);
+    DSN_SetDatabase(Connection);
 
     SQLDisconnect(Connection);
   }
@@ -696,6 +728,44 @@ INT_PTR SelectPath(HWND ParentWnd, int BoundEditId, const wchar_t *Caption, BOOL
 }
 
 
+BOOL CALLBACK addTooltips(HWND hwnd, LPARAM lParam)
+{
+  char className[256];
+  HWND hwndDlg= (HWND)lParam;
+  int  resourceId = GetDlgCtrlID(hwnd);
+  GetClassName(hwnd, className, sizeof(className));
+
+  if (resourceId != -1) //(resourceId == lblMaxKeyLengthTooltip)
+  {
+    if (_strnicmp(className, "Button", 6) == 0)//_strnicmp(className, "STATIC", 6) == 0)
+    {
+      int nChars= LoadString(hInstance, resourceId + TOOLTIP_TEXT_OFFSET, tipText, sizeof(tipText));
+
+      if (nChars > 0)
+      {
+        HWND hwndTip= CreateWindowEx(0, TOOLTIPS_CLASS, NULL, WS_POPUP | TTS_ALWAYSTIP | TTS_BALLOON, CW_USEDEFAULT, CW_USEDEFAULT,
+          CW_USEDEFAULT, CW_USEDEFAULT, hwndDlg, NULL, hInstance, NULL);
+
+        if (!hwndTip) {
+          return TRUE;
+        }
+        TOOLINFO toolInfo={0};
+        toolInfo.cbSize= sizeof(toolInfo);
+        toolInfo.hwnd= hwndDlg;
+        toolInfo.uFlags= TTF_IDISHWND | TTF_SUBCLASS;
+        toolInfo.uId= (UINT_PTR)hwnd;
+        toolInfo.lpszText= tipText;
+        //GetClientRect(hwnd, &toolInfo.rect);
+        SendMessage(hwndTip, TTM_ADDTOOL, 0, (LPARAM)&toolInfo);
+        SendMessage(hwndTip, TTM_ACTIVATE, TRUE, 0);
+        SetWindowPos(hwndTip, HWND_TOPMOST, 0, 0, 0, 0, SWP_NOMOVE | SWP_NOSIZE | SWP_NOACTIVATE);
+      }
+    }
+  }
+  return TRUE;
+}
+
+
 INT_PTR CALLBACK DialogDSNProc(HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
 {
   INT_PTR res;
@@ -704,16 +774,28 @@ INT_PTR CALLBACK DialogDSNProc(HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lPara
   {
   case WM_CTLCOLORDLG:
     if (!hbrBg)
-      hbrBg= CreateSolidBrush(RGB(255,255,255));
-      return (INT_PTR)hbrBg;
-    break;
-   case WM_CTLCOLORSTATIC:
-   {
-     HDC hdcStatic = (HDC)wParam;
-     SetTextColor(hdcStatic, RGB(0, 0, 0));
-     SetBkMode(hdcStatic, TRANSPARENT);
-     return (INT_PTR)hbrBg;
+    {
+      hbrBg= CreateSolidBrush(RGB(255, 255, 255));
+    }
+    return (INT_PTR)hbrBg;
+
+  case WM_CTLCOLORSTATIC:
+  {
+    HDC hdcStatic = (HDC)wParam;
+    SetTextColor(hdcStatic, RGB(0, 0, 0));
+    SetBkMode(hdcStatic, TRANSPARENT);
+    return (INT_PTR)hbrBg;
   }
+  /*case WM_NOTIFY:
+  {
+    LPNMHDR lpnmhdr = (LPNMHDR)lParam;
+    if (lpnmhdr->code == TTN_GETDISPINFO)
+    {
+      NMTTDISPINFO* pDispInfo = (NMTTDISPINFO*)lParam;
+      pDispInfo->lpszText = "----Short tooltip!----";
+    }
+    break;
+  }*/
   case WM_COMMAND:
     switch(LOWORD(wParam))
     {
@@ -721,15 +803,19 @@ INT_PTR CALLBACK DialogDSNProc(HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lPara
       SendMessage(GetParent(hDlg), WM_CLOSE, 0, 0);
       notCanceled= FALSE;
       return TRUE;
+
     case PB_PREV:
-	  SetPage(hDlg, -1);
-	  return TRUE;
+	    SetPage(hDlg, -1);
+	    return TRUE;
+
     case PB_NEXT:
-	  SetPage(hDlg, 1);
-	  return TRUE;
+	    SetPage(hDlg, 1);
+	    return TRUE;
+
     case pbTestDSN:
       MADB_WIN_TestDsn(TRUE);
-    return TRUE;
+      return TRUE;
+
 	  case cbDatabase:
     case cbCharset:
       if(HIWORD(wParam) == CBN_DROPDOWN)
@@ -742,34 +828,42 @@ INT_PTR CALLBACK DialogDSNProc(HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lPara
 	    return TRUE;
     case pbPlugindirBrowse:
       return SelectPath(hDlg, txtPluginDir,   L"Select Plugins Directory",          /* Folder*/ TRUE, TRUE);
+
     case pbCaPathBrowse:
       res=   SelectPath(hDlg, txtSslCaPath,   L"Select CA Path",                                TRUE, OpenCurSelection);
       OpenCurSelection= OpenCurSelection && !res;
       return res;
+
     case pbKeyBrowse:
       res=   SelectPath(hDlg, txtSslKey,      L"Select Client Private Key File",    /* File */  FALSE, OpenCurSelection);
       OpenCurSelection= OpenCurSelection && !res;
       return res;
+
     case pbCertBrowse:
       res=   SelectPath(hDlg, txtSslCert,     L"Select Public Key Certificate File",            FALSE, OpenCurSelection);
       OpenCurSelection= OpenCurSelection && !res;
       return res;
+
     case pbCaCertBrowse:
       res=   SelectPath(hDlg, txtSslCertAuth, L"Select Certificate Authority Certificate File", FALSE,OpenCurSelection);
       OpenCurSelection= OpenCurSelection && !res;
       return res;
+
     case pbServerKeyBrowse:
       res=   SelectPath(hDlg, txtServerKey, L"Select Server Public Key File", FALSE, OpenCurSelection);
       OpenCurSelection= OpenCurSelection && !res;
       return res;
+
     case pbFpListBrowse:
       res=   SelectPath(hDlg, txtTlsPeerFpList, L"Select File with SHA1 fingerprints of server certificates", FALSE, OpenCurSelection);
       OpenCurSelection= OpenCurSelection && !res;
       return res;
+
     case pbCrlBrowse:
       res = SelectPath(hDlg, txtCrl, L"Select PEM File Certificate Revocation List(CRL)", FALSE, OpenCurSelection);
       OpenCurSelection = OpenCurSelection && !res;
       return res;
+
     case rbTCP:
 	  case rbPipe:
 		  if (HIWORD(wParam) == BN_CLICKED)
@@ -792,6 +886,7 @@ INT_PTR CALLBACK DialogDSNProc(HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lPara
 	return FALSE;
 }
 
+
 INT_PTR CALLBACK DialogProc(HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
 {
   switch(uMsg)
@@ -824,6 +919,7 @@ INT_PTR CALLBACK DialogProc(HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
   	{
       hwndTab[i]= CreateDialog(hInstance, MAKEINTRESOURCE(Dialogs[i]), hDlg, DialogDSNProc);
 	    SetWindowPos(hwndTab[i], 0, 120, 5, (rc.right-rc.left)-120, (rc.bottom-rc.top), SWP_NOZORDER | SWP_NOACTIVATE);
+      EnumChildWindows(hwndTab[i], addTooltips, (LPARAM)hwndTab[i]);
 	    ShowWindow(hwndTab[i], (i == 0) ? SW_SHOW : SW_HIDE);
   	}
     i= 0;
@@ -833,8 +929,22 @@ INT_PTR CALLBACK DialogProc(HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
         MA_WIN_SET_MAXLEN(DsnMap[i].Page, DsnMap[i].Item, DsnMap[i].MaxLength);
       ++i;
     }
+    /*HWND hwndTip= CreateWindowEx(0, TOOLTIPS_CLASS, NULL, WS_POPUP | TTS_ALWAYSTIP | TTS_BALLOON, CW_USEDEFAULT, CW_USEDEFAULT,
+      CW_USEDEFAULT, CW_USEDEFAULT, hwndTab[0], NULL, hInstance, NULL);
+    HWND hwnd= GetDlgItem(hwndTab[0], lblDsn);
+    if (!hwndTip) {
+      return TRUE;
+    }
+    TOOLINFO toolInfo={0};
+    toolInfo.cbSize= sizeof(toolInfo);
+    toolInfo.hwnd= hwndTab[0];
+    toolInfo.uFlags= TTF_IDISHWND | TTF_SUBCLASS;
+    toolInfo.uId= (UINT_PTR)hwnd;
+    toolInfo.lpszText= "---- Short tooltip! ----";
+    SendMessage(hwndTip, TTM_ADDTOOL, 0, (LPARAM)&toolInfo);
+    SendMessage(hwndTip, TTM_ACTIVATE, TRUE, 0);
+    SetWindowPos(hwndTip, HWND_TOPMOST, 0, 0, 0, 0, SWP_NOMOVE | SWP_NOSIZE | SWP_NOACTIVATE);*/
     
-    return TRUE; 
   }
   }
   return FALSE;
@@ -878,7 +988,7 @@ BOOL DSNDialog(HWND     hwndParent,
 {
   MSG     msg;
   BOOL    ret;
-  char    *DsnName=  NULL;
+  const char *DsnName= NULL;
   my_bool DsnExists= FALSE;
   char    Delimiter= ';';
 
@@ -937,7 +1047,10 @@ BOOL DSNDialog(HWND     hwndParent,
         }
       }
       
+      /* These are defaults that driver appy */
       MADB_DSN_SetDefaults(Dsn);
+      /* These are defaults of the dialog */
+      DsnApplyDefaults(Dsn);
       MADB_ParseConnString(Dsn, (char *)lpszAttributes, SQL_NTS, Delimiter);
 
       /* Need to set driver after connstring parsing, and before saving */
@@ -995,10 +1108,14 @@ BOOL DSNDialog(HWND     hwndParent,
     }
   }
 
-  InitCommonControls();
+  INITCOMMONCONTROLSEX icc;
+  BOOL initResult= FALSE;
+  icc.dwSize = sizeof(INITCOMMONCONTROLSEX);
+  icc.dwICC = ICC_WIN95_CLASSES;
+  initResult= InitCommonControlsEx(&icc);
 
-  EffectiveDisabledPages=     DisabledPages[Dsn->isPrompt];
-  EffectiveDisabledControls=  DisabledControls[Dsn->isPrompt];
+  EffectiveDisabledPages=    DisabledPages[Dsn->isPrompt];
+  EffectiveDisabledControls= DisabledControls[Dsn->isPrompt];
 
   if (fRequest == ODBC_ADD_DSN && Dsn->isPrompt == MAODBC_CONFIG && Dsn->DSNName != NULL)
   {
@@ -1056,6 +1173,7 @@ BOOL INSTAPI ConfigDSN(
   case ODBC_ADD_DSN:
   case ODBC_CONFIG_DSN:
     memset(&Dsn, 0, sizeof(MADB_Dsn));
+    MADB_DSN_Init(&Dsn);
     return DSNDialog(hwndParent, fRequest, lpszDriver, lpszAttributes, &Dsn);
   case ODBC_REMOVE_DSN:
   {
